package edu.berkeley.cs160.teammoomoo.nutritrak160;

import android.app.Activity;
import android.os.Bundle;
import android.view.Menu;
import android.view.View;
import android.view.Menu.Item;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.ArrayAdapter;
import android.widget.AdapterView;
import android.widget.TextView;
import android.widget.EditText;
import android.widget.Toast;
import android.app.AlertDialog;
import android.content.Intent;

import java.util.*;

public class AddExerciseView extends Activity {
    /** Called when the activity is first created. */
	/**
	 * @author Fan
	 */
	
	private NutriTrakDdAdapter dbAdapter;
	
	private static final String[] mExercises = NutriTrakDdAdapter.EXERCISE_KEYS;
	private Spinner mSpinner;	// spinner for list of exercises
	private TextView dataText;	// tells what type of data is being entered (Distance, reps, etc)
	private EditText editData;	// lets user enter in the data
	private TextView unitText;	// Units for the exercise
	private TextView calequivText;	// Displays the estimated calories per unit exercise
	private Button moreButton;
	
	
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.addexercise);
        
        //set up the spinner
        mSpinner = (Spinner) findViewById(R.id.exercise_spinner);
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,
                android.R.layout.simple_spinner_item, mExercises);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mSpinner.setAdapter(adapter);
        

		//database initialization stuff
		this.dbAdapter = new NutriTrakDdAdapter(this);
		this.dbAdapter.open();
		
        dataText = (TextView) findViewById(R.id.data_type);
        editData = (EditText) findViewById(R.id.data_edit);
        unitText = (TextView) findViewById(R.id.unit_text);
        calequivText = (TextView) findViewById(R.id.cals_per_unit_text);
        moreButton = (Button) findViewById(R.id.add_more);
		
		
		// for changing items in the spinner
		mSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView parent, View v,
					int position, long id) {
				changeItemSelection();
			}

			@Override
			public void onNothingSelected(AdapterView arg0) {
				dataText.setVisibility(View.GONE);
				editData.setVisibility(View.GONE);
				unitText.setVisibility(View.GONE);
				moreButton.setVisibility(View.GONE);
			}
		});
		
		moreButton.setOnClickListener(new Button.OnClickListener() {
			public void onClick(View v) {
                addExerciseToDb();
                mSpinner.requestFocus();
            }
		});
		
    }
    
    // when the item selection is changed, then everything else needs to be changed
    private void changeItemSelection() {
    	String selected = (String) mSpinner.getSelectedItem();
        NutriTrakDdAdapter.ExerciseData exercise = this.dbAdapter.exerciseEntries.get(selected);
        
        // change the display data type
        if (exercise.type == NutriTrakDdAdapter.ExerciseType.DISTANCE) {
        	dataText.setText("Distance");
        }
        else if (exercise.type == NutriTrakDdAdapter.ExerciseType.REPETITIONS) {
        	dataText.setText("Repetitions");
        } else {
        	dataText.setText("Enter in Units");
        }
        
        String cals = Double.toString(exercise.calEquiv);
        
        // change the units
        if (exercise.unit == NutriTrakDdAdapter.ExerciseUnit.MILES) {
        	unitText.setText("Miles");
        	calequivText.setText(cals + " calories burned per mile");
        } else if (exercise.unit == NutriTrakDdAdapter.ExerciseUnit.YARDS) {
        	unitText.setText("Yards");
        	calequivText.setText(cals + " calories burned per yard");
        } else if (exercise.unit == NutriTrakDdAdapter.ExerciseUnit.REPETITIONS) {
        	unitText.setText("Reps");
        	calequivText.setText(cals + " calories burned per repetition");
        } else {
        	unitText.setText("Units");
        	calequivText.setText(cals + " calories burned each time");
        }
    }
    
    // add the exercise to the database based on what's currently on screen
    private void addExerciseToDb() {
    	String exerciseName = (String) mSpinner.getSelectedItem();
    	String durationString = (String) editData.getText().toString();
    	Integer duration;
    	try {
    		duration = Integer.parseInt(durationString);
    	} catch (NumberFormatException e){
    		duration = null;
    	}
    	
    	if (duration == null) {
    		AlertDialog.show(AddExerciseView.this,
    							null,
    							R.drawable.icon,
    							"Please enter in a value.",
    							"Ok",
    							true);
    		editData.setText("");
    		editData.requestFocus();
    	} else {
    		LinkedList<String> names = new LinkedList<String>();
    		LinkedList<Integer> durations = new LinkedList<Integer>();
    		names.add(exerciseName);
    		durations.add(duration);
    		Toast.makeText(this, "Saved exercise", Toast.LENGTH_SHORT).show();
    		dbAdapter.createExercises(names, durations);
    		mSpinner.setSelection(0);
            mSpinner.requestFocus();
            editData.setText("");
            changeItemSelection();
    	}
    }
    
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		return GlobalMenu.createContextMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, Item item) {
		return GlobalMenu.handleMenuItemSelect(featureId, item, this)?
				true:super.onMenuItemSelected(featureId, item);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, 
			String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		
		if(data != null && data.equalsIgnoreCase("chain")) {
			setResult(RESULT_OK, "chain");
			this.finish();
		}
	}
}