package edu.berkeley.cs160.teammoomoo.nutritrak160;

import java.io.IOException;
import java.net.MalformedURLException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.net.*;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.Random;
import java.util.ArrayList;
import java.util.LinkedList;

import org.apache.http.client.*;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;

import android.app.Activity;
import android.app.Dialog;
import android.database.Cursor;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.widget.*;
import android.view.Menu.Item;
import android.content.Context;
import android.content.Intent;
import android.app.DatePickerDialog;
import android.widget.DatePicker.OnDateSetListener;
import android.util.DateFormat;

public class HistoryView extends Activity {
	/**
	 * @author Ben, Brandon
	 * 
	 * See http://code.google.com/apis/chart/ for details of graph API
	 */
	private final static String baseUrl = "http://chart.apis.google.com/chart?";
	
	// UI crap
	private ImageView graph;
	private Button timeSetButton;
	private Button mDaySelectButton;
	private ListView nutrientNames;
	private String testQuery = "cht=p3&chd=t:60,40&chs=250x100&chl=Hello|World";
	
	// Database instance
	private NutriTrakDdAdapter mDbHelper = null;
	
	// State holding instance vars
	
	// Invariant: First element = year, second = month, 3rd = day
	private final LinkedList<Integer> yearMonthDay = new LinkedList<Integer>();
	
	@Override
	protected void onCreate(Bundle icicle) {
		super.onCreate(icicle);
        setContentView(R.layout.history);
        
        this.mDbHelper = new NutriTrakDdAdapter(this);
		this.mDbHelper.open();
		//	 TODO: don't forget to close this later
		
		this.nutrientNames = (ListView) findViewById(R.id.nutrientNames);  // or will it be meals etc?
		this.timeSetButton = (Button) findViewById(R.id.timeSetButton);
		this.graph = (ImageView) findViewById(R.id.graph);
		mDaySelectButton = (Button) findViewById(R.id.daySelectButton);
		
    	int year = Utils.getCurrentYear();
    	int monthOfYear = Utils.getCurrentMonth();
    	int dayOfMonth = Utils.getCurrentDayOfMonth();
    	this.setTime(monthOfYear, dayOfMonth, year);
    	
    	this.setTimeSetterListener();
    	setDaySelectListener();
    	this.fillNutrientsList(this.nutrientNames);
    	this.setNutrientsListener(this.nutrientNames);
    	this.nutrientNames.requestFocus();
    	this.nutrientNames.setSelection(0);
    	
		Log.d("HistoryView", "Done with initial setup");
	}
	
	private void setTime(int month, int day, int year) {
		this.yearMonthDay.clear();
		this.yearMonthDay.add(year);yearMonthDay.add(month);yearMonthDay.add(day);
		String monthName = Utils.getMonthName(month);
		this.setTitle(monthName+"-"+yearMonthDay.get(2)+"-"+yearMonthDay.get(0));
	}
	
	private void setNutrientsListener(final ListView list) {
		final HistoryView outerReference = this;
		
		list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView parent, View v, int position, long id) {
				// Show brief description of nutrient
				String nutrientName = (String) outerReference.nutrientNames.getSelectedItem();
				NutriTrakDdAdapter.NutrientEntry entry = outerReference.mDbHelper.nutrientEntries.get(nutrientName);;
				Toast.makeText(outerReference, entry.description, Toast.LENGTH_LONG).show();
			}
		});
		
		list.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

			public void onItemSelected(AdapterView parent, View v, int position, long id) {
				outerReference.refreshImage(outerReference.generateQuery());
				// We need to reset the adapter when we move the selector because it only will hold views
				// that were initially visible, unfortunately.  Reset will put in any newly visible views.
				// Problem: expensive to reset each time, esp in conjunction with network latency
				// Solution: invalidate only at key points, eg when you think you might see a new view
				// Such as when you're about to scroll to a view not visible.
				// Plan 1: 
				// Be simple, reset each time.  
				// Problem: race condition if hit button too fast
//				outerReference.fillNutrientsList(list);
//				list.setSelection(position);
				// Plan 2:
				// More efficient
				// Problem: Has 2 race conditions
				// Below: first and last views show on the screen in the listview
//				int first = list.getPositionForView(list.getChildAt(0));
//				int last = first + list.getChildCount(); 
//				Log.d("HistoryView", first+"");
//				Log.d("HistoryView", last+"");
//				
//				if (position < first+1 || position >= last-1) {
//					outerReference.fillNutrientsList(list);
//					list.setSelection(position);
//				}
			}

			public void onNothingSelected(AdapterView arg0) { /* Nothing */ }
		});
	}
	
	private void fillNutrientsList(ListView list) {
		String[] from = NutriTrakDdAdapter.STATS_TABLE_NUTRI_KEYS;
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, R.layout.text_row, from);
		list.setAdapter(adapter);
	}
	
	private String getSelectedNutrientName() {
		return (String) this.nutrientNames.getSelectedItem();
	}
	
	private void setTimeSetterListener() {
		final HistoryView outerReference = this;
		this.timeSetButton.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
            	OnDateSetListener l = new DatePicker.OnDateSetListener() {
					public void dateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
			        	outerReference.setTime(monthOfYear, dayOfMonth, year);
			        	outerReference.refreshImage(outerReference.generateQuery());
			        	outerReference.nutrientNames.requestFocus();
			        	outerReference.nutrientNames.setSelection(0);
					}
            	};
	        	int year = yearMonthDay.get(0);
	        	int monthOfYear = yearMonthDay.get(1);
	        	int dayOfMonth = yearMonthDay.get(2);
	        	int weekStartDay = Calendar.SUNDAY; 
	        	// Construct, show date dialog
	        	DatePickerDialog datePicker = new DatePickerDialog(outerReference, l, year, monthOfYear, dayOfMonth, weekStartDay);
	        	datePicker.show();
	        	// Can't refresh image here, must do it in the date set listener, show() does not block for user input
            }
        });
	}
	
	private void setDaySelectListener() {
		final HistoryView outerReference = this;
		this.mDaySelectButton.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
            	OnDateSetListener l = new DatePicker.OnDateSetListener() {
					public void dateSet(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
						Intent i = new Intent(outerReference, MainView.class);
						i.putExtra("year", year);
						i.putExtra("month", monthOfYear);
						i.putExtra("day", dayOfMonth);
						
						startSubActivity(i, GlobalMenu.MAINVIEW_ID);
					}
            	};
	        	int year = yearMonthDay.get(0);
	        	int monthOfYear = yearMonthDay.get(1);
	        	int dayOfMonth = yearMonthDay.get(2);
	        	int weekStartDay = Calendar.SUNDAY; 
	        	// Construct, show date dialog
	        	DatePickerDialog datePicker = new DatePickerDialog(outerReference, l, year, monthOfYear, dayOfMonth, weekStartDay);
	        	datePicker.show();
	        	// Can't refresh image here, must do it in the date set listener, show() does not block for user input
            }
        });
	}
	
	/*
	 * TODO:
	 * Generate a query based on nutrient name + time selected
	 * Not going to be easy, will involve going to db
	 * and creating a string to be interpreted by google charts API
	 * (Note that the aforementioned db methods have not been written yet)
	 */
	private String generateQuery() {
		String nutrientName = this.getSelectedNutrientName();
		/*
		 * Contains the values for a week in terms of nutrients
		 * First Sunday, then Monday, etc
		 */
		
		// line type chart 250x200 pixels 
		// TODO: labeling of 100 marker not working but a minor problem
		String query= "cht=lc&chs=250x200&chd=t:%s&chl=%s&chxt=y&chxr=%s&chm=%s&chxl=1:|dv&chxp=%s";
		String days = "Sun|Mon|Tue|Wed|Thu|Fri|Sat";
		String values = "";
		GregorianCalendar cal = new GregorianCalendar(yearMonthDay.get(0), yearMonthDay.get(1), yearMonthDay.get(2));
		Log.d("HistoryView", DateFormat.format("yyyy/MM/dd", cal).toString().replace('/', '-'));
		
		int[] data = mDbHelper.getStatsOfPeriod(cal, nutrientName, this);
		int maxVal = 0;
		for(int i=0; i < data.length; i++){
			maxVal = Math.max(maxVal, data[i]);
			values += String.format("%d.0%s", data[i], i < (data.length - 1) ? "," : "");
		}
		
		int peak = Math.max(100, maxVal);
		String range = "0,0,"+peak;
		float horMarkerStart = 100.0f/(float) peak;
		String marker100 = "r,ff0000,0,"+horMarkerStart+","+(horMarkerStart+0.01f);
		String DVstringPos = "1,100";
		
		String retVal = String.format(query, values, days, range, marker100, DVstringPos);
		
		return retVal;
	}
	
	private void refreshImage(String testQuery) {
		try {
			byte[] pngBytes = this.createPNGByteArray(testQuery);
			ByteArrayInputStream pngByteStream = new ByteArrayInputStream(pngBytes);
			BitmapDrawable drawableImage = new BitmapDrawable(pngByteStream);
			pngByteStream.close();
			this.graph.setImageDrawable(drawableImage);
		} catch (MalformedURLException e) {
			Log.d("HistoryView", "bad url format");
			Log.d("",e.toString());
		} catch (IOException e) {
			Log.d("HistoryView", "bad network");
			Log.d("",e.toString());
		} catch (Exception e) {
			Log.d("HistoryView", "um what the hell");
		}
	}
	
	/*
	 * Actually this is more of a utility method but whatever.
	 * Use apache to properly request PNG and get byte array back
	 * Using URLConnection and getInputStream won't work here
	 */
	private byte[] createPNGByteArray(String postUrl) {
		HttpClient client = new HttpClient();

		String url = baseUrl + postUrl;
		Log.d("HistoryView", url);
		HttpMethod method = null;

		method = new GetMethod(url);
		method.setRequestHeader("accept", "image/png");
		
		byte[] image = null;
		
		try {
			client.executeMethod(method);
			image = method.getResponseBody();
		} catch (Exception e) {
			Log.d("HistoryView", Utils.getStackTrace());
		} finally {
			method.releaseConnection();
			return image;
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		return GlobalMenu.createContextMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, Item item) {
		return GlobalMenu.handleMenuItemSelect(featureId, item, this)?
				true:super.onMenuItemSelected(featureId, item);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, 
			String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		
		if(data != null && data.equalsIgnoreCase("chain")) {
			setResult(RESULT_OK, "chain");
			this.finish();
		}
	}
}
