package edu.berkeley.cs160.teammoomoo.nutritrak160;

import java.util.*;

import android.app.Activity;
import android.app.LocalActivityManager;
import android.app.TabActivity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewInflate;
import android.view.Window;
import android.view.Menu;
import android.view.Menu.Item;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ScrollView;
import android.widget.SimpleCursorAdapter;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.LinearLayout.LayoutParams;
import android.util.DateFormat;
import android.util.Log;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.database.Cursor;

/**
 * The main view. Shows bar graph with current daily stats summary and has an
 * option to show full daily stats
 * @author Fan, Bruno
 */

public class MainView extends Activity {
	private static final String TAB_SUMMARY = "Summary";
	private static final String TAB_MEALS = "Meals";
	
	private NutriTrakDdAdapter mDbAdapter;
	private TabHost mTabContainer;
	private LinearLayout mTopLayout;
	private LinearLayout mBottomLayout;
	private ListView mMealsList;
	private ScrollView mScrollView;

	private GregorianCalendar mCurrentDate;

	private boolean mMain;
	
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.tabbed_main);

		//database initialization stuff
		this.mDbAdapter = new NutriTrakDdAdapter(this);
		this.mDbAdapter.createAndOpen();

		mTabContainer = (TabHost) findViewById(R.id.tabbedcontainer);
		
		mTabContainer.setup(new LocalActivityManager(this, true));
		
		TabHost.TabSpec summaryTab = mTabContainer.newTabSpec(TAB_SUMMARY);
		TabHost.TabSpec mealsTab = mTabContainer.newTabSpec(TAB_MEALS);
		
		summaryTab.setContent(R.id.scroll_view);
		mealsTab.setContent(R.id.main_view_bottom);

		summaryTab.setIndicator(new MyTabIndicator(this, "Summary"));
		mealsTab.setIndicator(new MyTabIndicator(this, "Meals"));
		
		mTabContainer.addTab(summaryTab);
		mTabContainer.addTab(mealsTab);
		
		mTabContainer.setOnTabChangedListener(new TabListener());
		
		
		mTopLayout = (LinearLayout) findViewById(R.id.main_view_top);
		mMealsList = (ListView) findViewById(R.id.meals_list);
		mBottomLayout = (LinearLayout) findViewById(R.id.main_view_bottom);
		mScrollView = (ScrollView) findViewById(R.id.scroll_view);
		
		//mScrollView.requestFocus();
		
		

		Bundle extras = icicle!=null? icicle:getIntent().getExtras();
		if(extras != null){
			int year = extras.getInt("year");
			int month = extras.getInt("month");
			int day = extras.getInt("day");
			
			mCurrentDate = new GregorianCalendar(year, month, day);
			mMain = false;
		}
		else {
			mCurrentDate = new GregorianCalendar();
			mMain = true;
		}
		
		fillBars(true);
		fillMeals();
		
		if(icicle!=null) 
			mTabContainer.setCurrentTab(icicle.getInt("selectedtab", 0));
		else
			mTabContainer.setCurrentTab(0);
		
		mMealsList.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView parent, View v, int position, 
					long id) {
				//((MealsListAdapter)parent.getAdapter()).toggle(position);
				if(id>0)
					mealSelected(id);

			}
		});
	}

	private void mealSelected(long mealId) {
		
		Intent i = new Intent(this, MealView.class);
    	i.putExtra(NutriTrakDdAdapter.KEY_MEAL_ID, (int) mealId);
    	startSubActivity(i, GlobalMenu.VIEWMEAL_ID); 
	}
	
	private void fillBars(boolean create) {
		//create new bars
		String[] nutrients = NutriTrakDdAdapter.STATS_TABLE_NUTRI_KEYS;

		Hashtable<String,Integer> totalStats = 
			this.mDbAdapter.getTotalStatsOfDay(mCurrentDate);

		ViewInflate vi= getViewInflate();
		
		for(int i = 0; i < nutrients.length; i++) {
			View newView;
			
			if(create)
				newView = vi.inflate(R.layout.progress_row, mTopLayout, false, null);
			else
				newView = mTopLayout.getChildAt(i);
			
			ProgressBar pb = (ProgressBar) newView.findViewById(R.id.progress);
			TextView tv = (TextView) newView.findViewById(R.id.progress_text);

			int maxProgress = pb.getMax();

			Integer total = totalStats.get(nutrients[i]);

			// get recommended daily values (not sure if this is correct)
			NutriTrakDdAdapter.NutrientEntry nutrient = 
				this.mDbAdapter.nutrientEntries.get(nutrients[i]);

			/* This logic is duplicated in several files.  I have created a more general
			 * version at NutriTrakDdAdapter. */
			
			// Retrieve the values from preferences
			boolean hasPref = false;
			int prefValue = -1;
			SharedPreferences prefs = 
				this.getSharedPreferences(PreferencesView.preferencesFileName, MODE_PRIVATE);
			Map thresholdPrefs = prefs.getAll();
			if (thresholdPrefs.containsKey(nutrients[i])) {
				hasPref = true;
				prefValue = (Integer) thresholdPrefs.get(nutrients[i]);
			}

			//calculate percentage
			float nutrientPercent = 0;
			if (nutrient != null) {
				if (nutrient.unit == NutriTrakDdAdapter.NutriUnit.GRAMS) {
					if (nutrient.gramsPerDV == -1) {
						// arbitrary
						nutrientPercent = (float) total / (hasPref ? prefValue : 100);
					} else {
						nutrientPercent = (float) total / (hasPref ? prefValue : nutrient.gramsPerDV);
					}
				} else if (nutrient.unit == NutriTrakDdAdapter.NutriUnit.SELF) {
					//assume calories
					nutrientPercent = (float) total / 
					(hasPref ? prefValue : NutriTrakDdAdapter.NutrientEntry.CaloriesTotal);
				} else {	// nutrientEntry.unit == DV
					nutrientPercent = (float) total / (hasPref ? prefValue : 100);
				}
			}

			double progressVal = ((int)(nutrientPercent*1000))/10.0;
			// set the text
			tv.setText(capitalize(nutrients[i]) + ": \n" 
					+ progressVal + "%");
			
			if (progressVal > 100.0 && progressVal < 110.0) {
				tv.setTextColor(android.graphics.Color.YELLOW);
			} else if (progressVal >= 110.0) {
				tv.setTextColor(android.graphics.Color.RED);
			} else if(progressVal > 90){
				tv.setTextColor(android.graphics.Color.GREEN);
			} else {
				tv.setTextColor(android.graphics.Color.LTGRAY);
			}

			//set the progress status			
			pb.setProgress((int) (nutrientPercent * maxProgress));

			if(create)
				//add to view
				mTopLayout.addView(newView);
		}
	}

	private void fillMeals() {
		int[] mealIds = mDbAdapter.getConsumedMealsOnDate(mCurrentDate);
		int[] exIds = mDbAdapter.getExercisesOnDate(mCurrentDate);

		MealsListAdapter adapter = 
			new MealsListAdapter(this, mealIds, exIds);
		// Set the adapter for the list that just requested to be filled
		mMealsList.setAdapter(adapter);
	}

	private String capitalize(String s) {
		return (s.length()>0) ? Character.toUpperCase(s.charAt(0)) + s.substring(1) : s;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		return GlobalMenu.createContextMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, Item item) {
		return GlobalMenu.handleMenuItemSelect(featureId, item, this)?
				true:super.onMenuItemSelected(featureId, item);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, 
			String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);

		if(mMain) {
			//if we are in the main screen, notify the menu
			GlobalMenu.backToMain();

			//update display if necessary
			switch(requestCode) {
			case GlobalMenu.ADDEXERCISE_ID:
			case GlobalMenu.CREATE_ID:
			case GlobalMenu.PREFERENCES_ID:
				fillBars(false);
				fillMeals();
				break;
			}
		}
		else {
			//check if we are supposed to go all the way back to the main screen
			if(data != null && data.equalsIgnoreCase("chain")) {
				setResult(RESULT_OK, "chain");
				this.finish();
			}
		}
	}

	@Override
	protected void onFreeze(Bundle outState) {
		
		outState.putInt("year", mCurrentDate.get(GregorianCalendar.YEAR));
		outState.putInt("month", mCurrentDate.get(GregorianCalendar.MONTH));
		outState.putInt("day", mCurrentDate.get(GregorianCalendar.DAY_OF_MONTH));
		outState.putInt("selectetab", mTabContainer.getCurrentTab());
		
		super.onFreeze(outState);
	}

	
	private class MealsListAdapter extends BaseAdapter {
		public MealsListAdapter(Context context, int[] mealIds, int[] exIds)
		{
			mContext = context;
			mMealIds = mealIds;
			mExIds = exIds;
			
			mExPos = 0;
			mMealPos = 0;
			mCount = mMealIds.length+mExIds.length;
			mIds = new int[mCount];
			mPicked = new boolean[mCount];
			
			for(int i=0; i < mCount; i++) {
				mPicked[i]=false;
			}
		}

		public int getCount() {
			return mCount;
		}

		public Object getItem(int position) {
			return mIds[position];
		}

		public long getItemId(int position) {
			return mIds[position];
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			String mTime;
			String exTime;
			int meal_id;
			boolean meal;
			
			if(mPicked[position]) {
				if(mIds[position]<0) {
					meal_id = -mIds[position]-1;
					meal = false;
				}
				else {
					meal_id = mIds[position];
					meal = true;
				}
			}
			else{
				mPicked[position] = true;
				if(mMealPos<mMealIds.length)
					mTime = mDbAdapter.getTimeOfConsumption(mMealIds[mMealPos]).substring(11, 16);
				else
					mTime = "25:00";
				if(mExPos<mExIds.length)
					exTime = mDbAdapter.getTimeExercised(mExIds[mExPos]).substring(11, 16);
				else
					exTime = "25:00";

				meal = false;

				int mh = Integer.parseInt(mTime.substring(0, 2));
				int eh = Integer.parseInt(exTime.substring(0, 2));
				if(mh < eh) {
					meal = true;
				}
				else if(mh==eh && Integer.parseInt(mTime.substring(3, 5)) <
						Integer.parseInt(exTime.substring(3, 5))) {
					meal = true;
				}

				if(meal) {
					meal_id = mMealIds[mMealPos++];
					mIds[position] = mMealIds[mMealPos-1];
				}
				else {
					meal_id = mExIds[mExPos++];
					mIds[position] = -mExIds[mExPos-1]-1;
				}
			}
			TextView mTitle = new TextView(mContext);
			String title;
			String time;
			String extra = "";

			if(meal){
				title = "Meal at ";
				time = mDbAdapter.getTimeOfConsumption(meal_id).substring(11, 16);;
			}
			else {
				Cursor cur = mDbAdapter.getExerciseInfo(meal_id);
				cur.first();
				time = cur.getString(0).substring(11, 16);

				//get duration units
				String type = cur.getString(2);
				NutriTrakDdAdapter.ExerciseData exercise = 
					mDbAdapter.exerciseEntries.get(type);

				String units;

				if (exercise.unit == NutriTrakDdAdapter.ExerciseUnit.MILES) {
					units = "miles";
				} else if (exercise.unit == NutriTrakDdAdapter.ExerciseUnit.YARDS) {
					units ="yards";
				} else if (exercise.unit == NutriTrakDdAdapter.ExerciseUnit.REPETITIONS) {
					units ="reps";
				} else {
					units ="units";
				}

				title = type+" at ";
				extra =" ("+cur.getInt(1)+" "+units+")";
			}

			int t =Integer.parseInt(time.substring(0, 2));
			if(t > 12) {
				time = (t-12)+time.substring(2)+"PM"; 
			} else if(t==12) {
				time = 12+time.substring(2)+"AM";
			} else {
				time=t+time.substring(2)+"AM";
			}

			mTitle.setText(title+time+extra);
			
			return mTitle;
		}

		private Context mContext;
		private int[] mMealIds;
		private int[] mExIds;
		private int mMealPos;
		private int mExPos;
		private int[] mIds;
		private int mCount;
		private boolean[] mPicked;
	}
	
    private class TabListener implements TabHost.OnTabChangeListener
    {
    	public void onTabChanged(String tabId)
    	{
    		if (tabId == null) { //corresponds to 1st tab (bug in Android)
    			mBottomLayout.setVisibility(View.GONE);
    			mScrollView.setVisibility(View.VISIBLE);
    			mScrollView.requestFocus();
    			setTitle("Daily Summary for: " 
    					+ DateFormat.format("MMMM dd, yyyy", mCurrentDate).toString());
    		}
    		else if (tabId.compareTo(TAB_MEALS) == 0) {
    			mScrollView.setVisibility(View.GONE);
    			mBottomLayout.setVisibility(View.VISIBLE);
    			setTitle("Meals for: " 
    					+ DateFormat.format("MMMM dd, yyyy", mCurrentDate).toString());
    			//mMealsList.setSelection(0);
    			//mMealsList.requestFocus();
    		}
    	}
    }
    
    public static class MyTabIndicator extends LinearLayout
	{
		public MyTabIndicator(Context context, String label)
		{
			super(context);

			View tab = ViewInflate.from(context)
			.inflate(R.layout.tab_indicator, this, null);

			TextView tv = (TextView)tab.findViewById(R.id.tab_label);
			tv.setText(label);
		}
	}
}