package edu.berkeley.cs160.teammoomoo.nutritrak160;

import java.util.Enumeration;
import java.util.Hashtable;

import edu.berkeley.cs160.teammoomoo.nutritrak160.NutriTrakDdAdapter.MealStruct;
import edu.berkeley.cs160.teammoomoo.nutritrak160.NutriTrakDdAdapter.NutrientEntry;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.ViewGroup;
import android.view.Menu.Item;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.content.Context;
import android.content.Intent;

/**
 * View to see meal details.  For now it shows time and date of the meal,
 * the items consumed and their quantities, and basic 
 * summary of major nutrient totals
 * 
 * @author Brandon
 *
 */

public class MealView extends Activity {
	public static final String TEXT_TEMPLATE = "";
	public static final int DEFAULT_MEAL = 1;
	public static final String MEAL_CONTENT_STR = 
		"Meal Contents:%s\n";
	public static final String MEAL_NUTRIENTS_STR= 
		"Nutritional Facts\n%s";
	private TextView meal_info;
	private TextView meal_nutrients;
	private int	mMealId;	
	private NutriTrakDdAdapter mDbHelper;
	private boolean mEditable;

	
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.meal);
        meal_info = (TextView) findViewById(R.id.meal_info);
        meal_nutrients = (TextView) findViewById(R.id.meal_nutrients);

        mDbHelper = new NutriTrakDdAdapter(this);
        mDbHelper.open();
        
        Button b = (Button) findViewById(R.id.view_meal_edit);
        Button bd = (Button) findViewById(R.id.view_meal_done);
        
        Bundle extras = icicle!=null? icicle:getIntent().getExtras();
        if (extras == null)
        {
        	Log.d("MealView", "Chose default meal");
        	mMealId = DEFAULT_MEAL;
        }
        
        else {
        	mMealId = extras.getInt(NutriTrakDdAdapter.KEY_MEAL_ID);
        	Log.d("MealView", "Retrieve from bundle: " + mMealId);
        	
        	/* why in the world would you want to _prevent_ the user from editing their
        	 * meals? There's no reason they should only have one chance to edit 
        	 * the meal...
        	 */
        	if(!extras.getBoolean("editable", false)) {
        		b.setVisibility(View.GONE);
        		bd.setVisibility(View.GONE);
        		findViewById(R.id.scroll_view).setLayoutParams(
        				new LinearLayout.LayoutParams(
        						ViewGroup.LayoutParams.FILL_PARENT, 
        						ViewGroup.LayoutParams.FILL_PARENT));
        		mEditable = false;
        	}
        	else
        		mEditable = true;
        }

        MealStruct meal = mDbHelper.getMealStruct(mMealId);
        String contents = "";
        String nutrients = "";
        for (int i = 0; i < meal.foodNames.length; i++){
        	contents += String.format("\n\t%s:\t%s %s", meal.foodNames[i], 
        			meal.quantities[i], "oz");
        }
        Hashtable<String, Integer> stats = mDbHelper.getStatsOfMeal(mMealId);
        meal_info.append(String.format(MEAL_CONTENT_STR, contents));
        
        Enumeration<String> e = stats.keys();
        while(e.hasMoreElements()){
        	String  nutrient = e.nextElement();
        	int     total = stats.get(nutrient);
        	NutrientEntry unit = mDbHelper.nutrientEntries.get(nutrient);
        	nutrients += String.format("\t%s: %s\n", nutrient, unit.convert(total));
        }
        meal_nutrients.append(String.format(MEAL_NUTRIENTS_STR, nutrients)); 
        
        
        b.setOnClickListener(new Button.OnClickListener()
        	{
        		public void onClick(View v) {
        			editMeal();
        		}
        	});
        
        
        bd.setOnClickListener(new Button.OnClickListener()
        	{
        		public void onClick(View v) {
        			die();
        		}
        	});
    }
    
    private void die() {
    	setResult(RESULT_OK, "chain");
    	finish();
    }
    
    private void editMeal() {
		Intent i = new Intent(this, SearchView.class);
		i.putExtra(NutriTrakDdAdapter.KEY_MEAL_ID, mMealId);
		i.putExtra(SearchView.KEY_CREATE_MEAL, true);
		i.putExtra(SearchView.KEY_EDIT_MEAL, true);
		startSubActivity(i, GlobalMenu.EDITMEAL_ID);
	}
    
	/* (non-Javadoc)
	 * @see android.app.Activity#onFreeze(android.os.Bundle)
	 */
	@Override
	protected void onFreeze(Bundle outState) {
		// TODO Auto-generated method stub
		outState.putInt(NutriTrakDdAdapter.KEY_MEAL_ID, mMealId);
		outState.putBoolean("editable", mEditable);
		
		super.onFreeze(outState);
	}

	/* (non-Javadoc)
	 * @see android.app.Activity#onKeyDown(int, android.view.KeyEvent)
	 */
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if(keyCode == KeyEvent.KEYCODE_BACK && mEditable) {
			editMeal();
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		return GlobalMenu.createContextMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, Item item) {
		return GlobalMenu.handleMenuItemSelect(featureId, item, this)?
				true:super.onMenuItemSelected(featureId, item);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, 
			String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		
		if(data != null && data.equalsIgnoreCase("chain")) {
			setResult(RESULT_OK, "chain");
			this.finish();
		}
	}
}
