package edu.berkeley.cs160.teammoomoo.nutritrak160;

import java.lang.System;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.text.DateFormat;
import java.util.*;

import android.app.Activity;
import android.app.Dialog;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.widget.*;
import android.view.ViewInflate;
import android.view.Menu.Item;
import android.content.Context;
import android.content.Intent;
import android.app.DatePickerDialog;
import android.widget.DatePicker.OnDateSetListener;
import android.widget.SimpleCursorAdapter.FilterQueryProvider;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;

public class PreferencesView extends Activity {
	/**
	 * @author Ben
	 * 
	 * We could also make this view more complicated later by adding a field
	 * for age, weight, and a spinner for male/female but unless we actually use
	 * these (dubious) it's not clear it makes any sense to add them other
	 * then just because we can.
	 */
	
	private ListView myThresholdList;
	private NutriTrakDdAdapter myDB;
	private Button myResetButton;
	public final static String preferencesFileName = "thresholds";
	
	@Override
	protected void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.preferences);
		
		//database initialization stuff
		myDB = new NutriTrakDdAdapter(this);
		myDB.open();
		
		myThresholdList = (ListView) findViewById(R.id.threshold_list);
		fillThresholdList();
		setThresholdListener();
		
		myResetButton = (Button) findViewById(R.id.threshold_reset);
		setResetListener();
	}
	
	public void setResetListener() {
		final PreferencesView outerReference = this;
		myResetButton.setOnClickListener(new Button.OnClickListener() { 
			 public void onClick(View v) {
				Log.d("PreferencesView", "Resetting the thresholds");
				SharedPreferences prefs = outerReference.getSharedPreferences("thresholds", MODE_PRIVATE);
				Editor editor = prefs.edit();
				editor.clear();
				Utils.assertTrue(editor.commit());
			 }
		});
	}
	
	public void fillThresholdList() {
		String[] nutri_keys = NutriTrakDdAdapter.STATS_TABLE_NUTRI_KEYS.clone();
		for (int i = 0; i < NutriTrakDdAdapter.STATS_TABLE_NUTRI_KEYS.length; i++) {
			String nutrient = nutri_keys[i];
			nutri_keys[i] += ": " + getCurrentQuantity(nutrient) + " " + getNutrientType(nutrient);
		}
		final String[] from = nutri_keys;
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, R.layout.text_row, from);
		myThresholdList.setAdapter(adapter);
	}
	
	public void setThresholdListener() {
		final PreferencesView outerReference = this;
		myThresholdList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			public void onItemClick(AdapterView parent, View v, int position,
					long id) {
				String nutriKey = (String) outerReference.myThresholdList.getSelectedItem();
				showThresholdDialog(nutriKey);
			}
		});
	}
	
	private int getCurrentQuantity(final String nutriKey) {
		NutriTrakDdAdapter.NutrientEntry entry = myDB.nutrientEntries.get(nutriKey);
		SharedPreferences prefs = this.getSharedPreferences(preferencesFileName, MODE_PRIVATE);
		Map thresholdPrefs = prefs.getAll();
		
		/*
		 * If preferences object already has pref, use that otherwise default to
		 * nutriententries hashtable
		 */
		final int initialQuantity;
		if (thresholdPrefs.containsKey(nutriKey)) {
			int possible = (Integer) thresholdPrefs.get(nutriKey);
			if (possible == -1)
				// degenerate case where nutrient is unnecessary, set some arbitrary default
				initialQuantity = 100;
			else
				initialQuantity = possible;
		} else if (entry.unit == NutriTrakDdAdapter.NutriUnit.SELF) {
			initialQuantity = NutriTrakDdAdapter.NutrientEntry.CaloriesTotal;
		} else if (entry.unit == NutriTrakDdAdapter.NutriUnit.GRAMS) {
			if (entry.gramsPerDV == -1) 
				initialQuantity = 100;  // degenerate case, set some arbitrary default
			else
				initialQuantity = entry.gramsPerDV;
		} else {
			initialQuantity = 100;
		}
		
		return initialQuantity;
	}
	
	private String getNutrientType(final String nutriKey) {
		NutriTrakDdAdapter.NutrientEntry entry = myDB.nutrientEntries.get(nutriKey);
		if (entry.unit == NutriTrakDdAdapter.NutriUnit.DAILY_VALUE)
			return ("DV");
		if (entry.unit == NutriTrakDdAdapter.NutriUnit.GRAMS)
			return ("grams");
		if (entry.unit == NutriTrakDdAdapter.NutriUnit.SELF) {
			Utils.assertTrue(nutriKey.equals("calories"));
			return ("calories");
		} else {
			Utils.assertNotReached();
			return "";
		}
	}
	
	// A bit messy because of all the conversion factors and error checking
	private void showThresholdDialog(final String thresholdString) {
		// something like calories: 200 grams  <--- we want to grab back the calories
		int ind = thresholdString.indexOf(':');
		final String nutriKey = thresholdString.substring(0, ind);
		Log.d("PreferencesView", "showThresholdDialog()");
		
		final Dialog d = new Dialog(this);
		d.setTitle("Enter quantity");
		d.setContentView(R.layout.quantity_dialog);
		
		final EditText quantityText = (EditText) d.findViewById(R.id.quantitytext);
		quantityText.requestFocus();
		final TextView unitText = (TextView) d.findViewById(R.id.unit_text);
		unitText.setText(getNutrientType(nutriKey));
		
		final int initialQuantity = getCurrentQuantity(nutriKey);
		
		quantityText.setHint(Integer.toString(initialQuantity));
		
		final PreferencesView outerReference = this;
		quantityText.setOnKeyListener(new View.OnKeyListener() {
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if(keyCode == KeyEvent.KEYCODE_DPAD_CENTER && event.getAction() == KeyEvent.ACTION_DOWN ) {
					Log.d("PreferencesView", "Picked a quantity!");
					String enteredText = quantityText.getText().toString();
					int quantity = 0;
					if (!enteredText.equals("")) {
						quantity = Integer.parseInt(enteredText);
						// edit shared preferences only if change
						if (initialQuantity != quantity) {
							SharedPreferences prefs = outerReference.getSharedPreferences(preferencesFileName, MODE_PRIVATE);
							Editor editor = prefs.edit();
							editor.putInt(nutriKey, quantity);
							Utils.assertTrue(editor.commit());
			            	Toast.makeText(outerReference, "Saved preference", Toast.LENGTH_SHORT).show();
			            	outerReference.fillThresholdList();
						}
						d.dismiss();
						return true;
					}
					Toast.makeText(outerReference, "Please enter a value", Toast.LENGTH_SHORT).show();
					return true;
				}
				return false;
			}

		});

		d.show();
		
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		return GlobalMenu.createContextMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, Item item) {
		return GlobalMenu.handleMenuItemSelect(featureId, item, this)?
				true:super.onMenuItemSelected(featureId, item);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, 
			String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		
		if(data != null && data.equalsIgnoreCase("chain")) {
			setResult(RESULT_OK, "chain");
			this.finish();
		}
	}
}
