package edu.berkeley.cs160.teammoomoo.nutritrak160;

import java.lang.System;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.LinkedList;

import edu.berkeley.cs160.teammoomoo.nutritrak160.NutriTrakDdAdapter.MealStruct;

import android.app.Activity;
import android.app.Dialog;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.ViewGroup;
import android.widget.*;
import android.view.ViewInflate;
import android.view.Menu.Item;
import android.content.Context;
import android.content.Intent;
import android.app.DatePickerDialog;
import android.widget.DatePicker.OnDateSetListener;
import android.widget.SimpleCursorAdapter.FilterQueryProvider;

/**
 * 
 * View for searching for foods. Has a cascading columns categorical 
 * search and an auto complete search. Serves as the view for both
 * search for food info and for creating meals.  The category search
 * will be composed of 2 or 3 lists. 
 * extras bundle options:
 * KEY_CREATE_MEAL (boolean): specifies if the meal cart should be shown
 * and if the user should be allowed to create a new meal
 * 
 * @author Bruno, Ben
 *
 */


 // TODO: fix focus path


public class SearchView extends Activity {
	//Bundle keys
	public static final String KEY_CREATE_MEAL= "create";
	public static final String KEY_EDIT_MEAL = "edit";

	// database instance
	private NutriTrakDdAdapter mDbHelper;

	//UI elements
	private LinearLayout mListContainer;
	private LinearLayout mFoodCart;
	private ListView mSelectedFoodList;
	private Button mDoneButton;
	private ListView mList2;
	private ListView mList3;
	private AutoCompleteTextView mAutoSearch;

	/**
	 * An array of currently existing lists. only the last two 
	 * or three are shown at a time.  There is always at least on list
	 * in the array and three lists precreated, alll other lists are
	 * created dynamically
	 */
	private ArrayList<ListView> mLists = new ArrayList<ListView>(5);

	/** true when we are creating a new meal or editing an existing meal */
	private boolean mCreateMeal = true;
	
	/** true when we are editing an existing meal */
	private boolean mEditMeal = true;
	
	/** if we are editing a meal, contains the meal id to edit */
	private int mMealId = 0;

	/**set to true when the currently focused list is compromised of foods**/
	private boolean mFoodList = false;
	
	/** Instance vars for currently held foods and quantities in cart */
	private LinkedList<Integer> mFoodIDs;
	private LinkedList<Integer> mQuantities;
		
	@Override
	/** Called when the activity is first created. */
	public void onCreate(Bundle icicle) {
		//setTheme(android.R.style.Theme_Light);
		super.onCreate(icicle);
		setContentView(R.layout.search);
		this.setTitle("NutriTrak160-Search");

		//get bundle items
		Bundle extras = getIntent().getExtras();
		
		if(extras!=null) {
			mCreateMeal = extras.getBoolean(KEY_CREATE_MEAL);
			mEditMeal = extras.getBoolean(KEY_EDIT_MEAL);
			if (mEditMeal){
				mMealId = extras.getInt(NutriTrakDdAdapter.KEY_MEAL_ID);
			}
		}
		
		//database initialization stuff
		this.mDbHelper = new NutriTrakDdAdapter(this);
		this.mDbHelper.open();
		//	 TODO: don't forget to close this later

		//auto complete initialization
		this.mAutoSearch = (AutoCompleteTextView) findViewById(R.id.search_entry);
		this.fillDataAutoComplete();
		this.setAutoCompleteListener();

		//categorical search initialization
		mListContainer = (LinearLayout) findViewById(R.id.category_lists);

		ListView list0 = (ListView) findViewById(R.id.search_list1);
		mLists.add(list0);
		setListeners(list0);
		fillData(0, 0);

		mList2 = (ListView) findViewById(R.id.search_list2);
		mList3 = (ListView) findViewById(R.id.search_list3);
		setListeners(mList2);
		setListeners(mList3);

		mFoodCart = (LinearLayout) findViewById(R.id.food_cart);
		mSelectedFoodList = (ListView) findViewById(R.id.food_cart_list);
		mDoneButton = (Button) findViewById(R.id.create_meal_done);
		
		this.mFoodIDs = new LinkedList<Integer>();
		this.mQuantities = new LinkedList<Integer>();
		
		this.setDoneButtonListener(mDoneButton);

		//make sure that the other two lists are invisible and one gone
		if(mCreateMeal) {
			mList2.setVisibility(View.INVISIBLE);
			mList3.setVisibility(View.GONE);
			mFoodCart.setVisibility(View.VISIBLE);
		} else {
			mList2.setVisibility(View.INVISIBLE);
			mList3.setVisibility(View.INVISIBLE);
			mFoodCart.setVisibility(View.GONE);
		}
		
		if (mEditMeal){
			Log.d("SearchView", "filling cart");
			fillCart();
		}
		
		mSelectedFoodList.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView parent, View v, int position,
					long id) {
				int quantity = mQuantities.get(position);
				id = mFoodIDs.get(position).longValue();
				Log.d("SearchView", ""+quantity);
				Log.d("SearchView", ""+id);
				
				showQuantityDialog(id, quantity, true);
			}
		});
		

	}
	
	/**
	 * fills the shopping cart based on the current meal id
	 */
	
	private void fillCart()
	{
		/* get the meal */
		MealStruct m = mDbHelper.getMealStruct(mMealId);
		for (int i = 0; i < m.foodIDs.length; i++)
		{
			addFoodToCart(m.foodIDs[i], m.quantities[i], false);
		}
		fillDataShoppingList();
		
	}

	/**
	 * creates a new list based on the first list in the list of lists.
	 * the list will be added to the layout,
	 * and will have the listners set
	 * 
	 * @return returns the newly created list
	 */
	private ListView createList() {
		//get the layout params
		android.widget.LinearLayout.LayoutParams params = 
			(android.widget.LinearLayout.LayoutParams)
			mLists.get(0).getLayoutParams();

		ListView newList = new ListView(this);
		//set the layout params
		newList.setLayoutParams(params);
		//add it to the view
		mListContainer.addView(newList);
		//set listeners
		setListeners(newList);

		return newList;
	}

	/**
	 * sets the  listeners for a list so that it behaves properly
	 * @param thisList the list to receive the listeners
	 */
	private void setListeners(final ListView thisList) {
		final AdapterView.OnItemClickListener listener = new AdapterView.OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView parent, View v, int position,
					long id) {
				
				if(mFoodList) {
					if(mCreateMeal) {
						showQuantityDialog(id, 0);
					} else {
						Intent i = new Intent(SearchView.this, FoodView.class);
						i.putExtra(NutriTrakDdAdapter.KEY_FOOD_ID, (int) id);
						startSubActivity(i, 0);
					}
					
					return;
				}
				//category list so open the next list
				
				int size = mLists.size();
				ListView list;

				switch(size) {
				case 1:
					list = mList2;
					break;
				case 2:
					if(!mCreateMeal) {
						list = mList3;
						break;
					}
				default:
					list= createList();
				mLists.get(size-(mCreateMeal?2:3)).setVisibility(View.GONE);
				}

				list.setVisibility(View.VISIBLE);
				mLists.add(list);

				//fill new list with appropriate data
				if(!fillData(size, id)) {
					removeTopList();
					return;
				}

				list.requestFocus();
				list.setSelection(0);

				//color the selected item
				v.setBackground(R.drawable.yellow);
			}
		};
		thisList.setOnItemClickListener(listener);

		thisList.setOnKeyListener(new View.OnKeyListener() {
			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				int level = mLists.size();
				
				if(keyCode == KeyEvent.KEYCODE_DPAD_LEFT 
						&& event.getAction() == KeyEvent.ACTION_DOWN
						&& level == 1 && !mCreateMeal) {
					// Otherwise we'll lose focus if we try to go left when on root in lookupfoods
					mLists.get(0).getSelectedView().setBackground(R.drawable.transparent_background);
					return true;
				} else if(keyCode == KeyEvent.KEYCODE_DPAD_LEFT 
						&& event.getAction() == KeyEvent.ACTION_DOWN
						&& level != 1) {
					//go back one list to the previous
					removeTopList();
					return true;
				} else if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT 
						&& event.getAction() == KeyEvent.ACTION_DOWN) {
					if (!mFoodList) {
						// Emulate the "click" event, dispatch it
						// Now the d-pad right key should work!!!  
						listener.onItemClick(thisList, 
											thisList.getSelectedView(), 
											thisList.getSelectedItemPosition(), 
											thisList.getSelectedItemId());
					}
					return true;
					
					/* the following crap just consumes key events that would allow the user to
					 * use the selector as a means to switch focus while deep in category view,
					 * which breaks our category view (eg leaves selectors lying around)
					 */
				} else if (keyCode == KeyEvent.KEYCODE_DPAD_UP 
						&& event.getAction() == KeyEvent.ACTION_DOWN) {
					
					int selectorPos = thisList.getSelectedItemPosition();
					if (mLists.get(0) != thisList && selectorPos <= 0) {
						return true;
					}
					return false;
					
				}else if (keyCode == KeyEvent.KEYCODE_DPAD_DOWN
						&& event.getAction() == KeyEvent.ACTION_DOWN) {
					
					int selectorPos = thisList.getSelectedItemPosition();
					if (mLists.get(0) != thisList && selectorPos >= thisList.getCount()-1) {
						return true;
					}
					return false;
					
				}

				return false;
			}
		});
	}
	
	private void removeTopList() {
		ListView list;
		int level = mLists.size();
		
		switch(level) {
		case 2:
			mLists.get(1).setVisibility(View.INVISIBLE);
			break;
		case 3:
			if(!mCreateMeal) {
				mLists.get(2).setVisibility(View.INVISIBLE);
				break;
			}
		default:
			list = mLists.get(level-1);
		mListContainer.removeView(list);
		mLists.get(level-(mCreateMeal?3:4)).setVisibility(View.VISIBLE);
		}

		mLists.remove(level-1);
		list = mLists.get(level-2);
		list.requestFocus();

		//change color back
		list.getSelectedView().setBackground(R.drawable.transparent_background);
		if (mCreateMeal)
			list.setBackground(R.drawable.transparent_background);
		
		//set food list to false
		mFoodList = false;
	}
	
	/**
	 * Set the done button so that when clicked it will record all the selected foods + quantities
	 * and commit this data as a meal to the database
	 */
	private void setDoneButtonListener(Button doneButton) {
		final SearchView outerReference = this;
		this.mDoneButton.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
            	long meal_id;
            	if (!mEditMeal){
            		meal_id = outerReference.mDbHelper.createMeal(
            			outerReference.mFoodIDs, outerReference.mQuantities);
            	}
            	else {
            		meal_id = mMealId;
            		outerReference.mDbHelper.updateMeal(meal_id, 
            			outerReference.mFoodIDs, outerReference.mQuantities);
            		
            	}
            	
            	Toast.makeText(outerReference, "Saved meal", Toast.LENGTH_SHORT).show();
            	outerReference.mFoodIDs.clear();
            	outerReference.mQuantities.clear();
            	
            	Intent i = new Intent(outerReference, MealView.class);
            	i.putExtra(NutriTrakDdAdapter.KEY_MEAL_ID, (int) meal_id);
            	i.putExtra("editable", true);
            	startSubActivity(i, GlobalMenu.VIEWMEAL_ID);         	
            }
        });
		/*
		 * Prevent the user from focusing on an empty food cart, which causes focus to disappear
		 */
		this.mDoneButton.setOnKeyListener(new View.OnKeyListener() {
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if(keyCode == KeyEvent.KEYCODE_DPAD_UP 
						&& event.getAction() == KeyEvent.ACTION_DOWN 
						&& mCreateMeal) {
					if (outerReference.mFoodIDs.isEmpty()) {
						outerReference.mAutoSearch.requestFocus();
						return true;
					}
				}
				return false;
			}
		});
	}
	
	/**
	 * For now fills a list with the given array
	 * @param listNum the index of the list in the list of lists
	 * @param from the array to fill the list with
	 */
	private boolean fillData(int listNum, long categoryId) {	
		//special case for first list because of favorites
		if(listNum == 0 && categoryId==0) {
			int[] ids  = mDbHelper.getTopLevelCategories();
			
			FirstColumAdapter adapter = 
				new FirstColumAdapter(this, ids);
			
			mLists.get(listNum).setAdapter(adapter);
			mFoodList = false;
			return true;
		}
		
		Cursor cur; 
		
		if(categoryId==-1) {
			cur = mDbHelper.getFavoriteFoods(10);
			mFoodList = true;
		}else {
			cur = mDbHelper.getCategoryChildrenOfCategory(categoryId);
			
			// If the cursor is empty, means category has no category children
			// Therefore must be food, retrieve food children cursor
			if(!cur.first()) {
				cur = mDbHelper.getFoodChildrenOfCategory(categoryId);
				mFoodList = true;
			}
		}
		
		//check if we have an empty category
		if(!cur.first()) {
			Toast.makeText(this, "Empty Category", Toast.LENGTH_SHORT).show();
			return false;
		}
		
		startManagingCursor(cur);
		
		String[] from = new String[]{NutriTrakDdAdapter.KEY_NAME};
		int[] to = new int[]{R.id.text_row};
		
		SimpleCursorAdapter adapter = 
			new SimpleCursorAdapter(this, 
					mFoodList?R.layout.text_row:R.layout.text2_row, 
							cur, from, to);
		// Set the adapter for the list that just requested to be filled
		mLists.get(listNum).setAdapter(adapter);
		return true;
	}

	private class FirstColumAdapter extends BaseAdapter {

		private int[] mIds;
		private Context mContext;
		
		public FirstColumAdapter(Context c, int[] ids) {
			mContext = c;
			mIds = ids;
		}
		
		@Override
		public int getCount() {
			return mIds.length+1;
		}

		@Override
		public Object getItem(int position) {
			if(position == 0)
				return -1;
			
			return mIds[position-1];
		}

		@Override
		public long getItemId(int position) {
			if(position == 0)
				return -1;
			
			return mIds[position-1];
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			
			ViewInflate vi = getViewInflate();
			
			View v = vi.inflate(R.layout.text2_row, parent, false, null);
			TextView tv = (TextView) v.findViewById(R.id.text_row);
			
			if(position == 0) {
				tv.setText(R.string.favorites);
			} else {
				String text = mDbHelper.getCategoryName(mIds[position-1]);
				tv.setText(text);
			}
			
			return v;
		}
		
		
	}
	
	/**
	 * Sets the adapter for the autocomplete text view
	 */
	private void fillDataAutoComplete() {
		// Select the column that the autocomplete is filtering
		String[] columns = new String[] {NutriTrakDdAdapter.KEY_NAME};
		// Use this text row to display the autocomplete messages
		int[] views = {R.id.text_row};
		// Initial cursor (probably just a formality)
		Cursor foodNames = this.mDbHelper.getFoods();
		startManagingCursor(foodNames);
		SimpleCursorAdapter adapter = 
			new SimpleCursorAdapter(this, R.layout.text_row, foodNames, columns, views);
		// We want the NAME column (index = 1) to be used as the string representation of the cursor's entries
		adapter.setStringConversionColumn(1);
		adapter.setFilterQueryProvider(new FoodNameFilter(this.mDbHelper));
		this.mAutoSearch.setAdapter(adapter);
	}
	
	/**
	 * Sets the autocomplete listener to show quantity dialog at appropriate
	 * time (when center button pressed)
	 */
	private void setAutoCompleteListener() {
		final SearchView outerReference = this;
		this.mAutoSearch.setOnKeyListener(new View.OnKeyListener() {
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				// need to specify that the KeyEvent is action down, otherwise the first few moments of press
				// will translate into an event, and be consumed, but the leftover will create another event! 
				if(keyCode == KeyEvent.KEYCODE_DPAD_CENTER && event.getAction() == KeyEvent.ACTION_DOWN ) {
					// If press down, autocomplete with whatever is currently selected if popup showing
					outerReference.mAutoSearch.performCompletion();
					// User may have tried to trick us, might not be food (random garbage)
					String potentialFoodName = outerReference.mAutoSearch.getText().toString();
					NutriTrakDdAdapter db = outerReference.mDbHelper;
					if (db.isFood(potentialFoodName)) {
						Log.d("SearchView", "Found a real food");
						if(mCreateMeal) {
							showQuantityDialog(db.fromFoodNameToID(potentialFoodName),0);
							outerReference.mAutoSearch.setText("");
						} else {
							Intent i = new Intent(SearchView.this, FoodView.class);
							i.putExtra(NutriTrakDdAdapter.KEY_FOOD_ID, (int) db.fromFoodNameToID(potentialFoodName));
							outerReference.mAutoSearch.setText("");
							startSubActivity(i, 0);
						}
					} else {
						// Notify user that this food doesn't exist using unobtrusive message
						Log.d("SearchView", "Bad food name");
						Toast.makeText(outerReference, "No such food exists", Toast.LENGTH_SHORT).show();
					}
					return true;
				} else if(keyCode == KeyEvent.KEYCODE_DPAD_DOWN && event.getAction() == KeyEvent.ACTION_DOWN) {
					// This is to fix focus.  Want to go to categorical search from autocomplete.
					// Not a complete fix, only works when empty
					// Problem: method for telling whether autocomplete list is showing is a protected method
					String potentialFoodName = outerReference.mAutoSearch.getText().toString();
					if (potentialFoodName.equals("")) {
						outerReference.mLists.get(0).requestFocus();
						return true;
					}
				}
				return false;
			}
		});
	}
	
	private void fillDataShoppingList() {
		LinkedList<String> tempList = new LinkedList<String>();
		for (int i = 0; i < this.mFoodIDs.size(); i++) {
			String foodName = this.mDbHelper.fromFoodIDtoName(mFoodIDs.get(i));
			int quantity = this.mQuantities.get(i);
			tempList.add(quantity + " " + foodName);
		}
		final String[] from = tempList.toArray(new String[]{});
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, R.layout.foodcart_row, from);
		mSelectedFoodList.setAdapter(adapter);
		mSelectedFoodList.setFocusable(this.mFoodIDs.size()!=0);
	}

	private class FoodNameFilter implements FilterQueryProvider{
		NutriTrakDdAdapter dbAdapter;

		public FoodNameFilter (NutriTrakDdAdapter dbAdapter) {
			this.dbAdapter = dbAdapter;
		}

		public Cursor runQuery(CharSequence constraint) {
			Log.d("SearchView", "The constraint imposed on autocomplete is " + constraint);
			// Since return cursor, db must perform the filter itself (can't do in Java)
			return this.dbAdapter.getFoods(constraint);
		}
	}
	
	// replace = substitute whatever is already in shopping list
	// eg quantity = 0, delete
	// otherwise add to list
	private void addFoodToCart(long id, int quantity, boolean replace) {
		if (!replace) {
			// Might still replace tho if ID already exists in shopping list
			if (quantity != 0) {
				int index = mFoodIDs.indexOf((int) id);
				if (index != -1) {
					// Means already present, add to quantity already there
					mQuantities.set(index, mQuantities.get(index)+quantity);
				} else {
					mFoodIDs.add((int)id);
					mQuantities.add(quantity);
				}
			}
		} else {
			// replace
			if (quantity == 0) {
				int index = mFoodIDs.indexOf((int) id);
				mFoodIDs.remove(index);
				mQuantities.remove(index);
			} else {
				int index = mFoodIDs.indexOf((int) id);
				mQuantities.set(index, quantity);
			}
		}
	}
	
	private void showQuantityDialog(final long foodId, final int initialQuantity) {
		showQuantityDialog(foodId, initialQuantity,  false);
	}
	
	private void showQuantityDialog(final long foodId, final int initialQuantity, final boolean replace) {
		final Dialog d = new Dialog(this);
		d.setTitle("Enter quantity");
		d.setContentView(R.layout.quantity_dialog);
		
		final EditText quantityText = (EditText) d.findViewById(R.id.quantitytext);
		// Only show quantity if non-zero, so that user doesn't have to delete 0 (usability issue)
		if (initialQuantity != 0) 
			quantityText.setHint(Integer.toString(initialQuantity));
		quantityText.requestFocus();
		
		final TextView unitText = (TextView) d.findViewById(R.id.unit_text);
//		unitText.setText(this.mDbHelper.getFoodUnits((int) foodId));
		// Units field deprecated, but waste of time to remove it from everywhere in code, for now
		unitText.setText("ounces");

		quantityText.setOnKeyListener(new View.OnKeyListener() {
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if(keyCode == KeyEvent.KEYCODE_DPAD_CENTER && event.getAction() == KeyEvent.ACTION_DOWN ) {
					String enteredText = quantityText.getText().toString();
					int quantity = 0;
					if (!enteredText.equals("")) {
						quantity = Integer.parseInt(enteredText);
					}
					// if "", then default to 0.  parseInt won't throw exception, line doesn't allow non-numbers
					addFoodToCart(foodId, quantity, replace);
					fillDataShoppingList();
					d.dismiss();
					return true;
				}
				return false;
			}

		});

		d.show();
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		return GlobalMenu.createContextMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, Item item) {
		return GlobalMenu.handleMenuItemSelect(featureId, item, this)?
				true:super.onMenuItemSelected(featureId, item);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, 
			String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);

		if(data != null && data.equalsIgnoreCase("chain")) {
			setResult(RESULT_OK, "chain");
			this.finish();
		}
		
		//if we were creating a meal don't want user to come back to the creation of a saved meal
		else if(mCreateMeal) {
			setResult(RESULT_OK);
			this.finish();
		}
		
		else {
			int size = mLists.size();
			for(int i = size-1; i >0; i--) {
				ListView list = mLists.get(i);
				if(i>2)
					mListContainer.removeView(list);
				else
					list.setVisibility(View.INVISIBLE);
				mLists.remove(i);
				mFoodList = false;
			}
			ListView list =mLists.get(0);
			list.getSelectedView().setBackground(R.drawable.transparent_background);
			list.setVisibility(View.VISIBLE);
		}
	}
}