package edu.berkeley.cs160.teammoomoo.nutritrak160;

import android.os.SystemClock;
import android.util.TimeUtils;
import android.app.Activity;
import android.app.TimePickerDialog;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import android.view.Menu;
import android.view.Menu.Item;

import java.io.*;
import java.util.LinkedList;
import java.lang.System;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import java.util.SimpleTimeZone;
import java.text.DateFormat;
import java.sql.Time;
import java.sql.Date;
import java.sql.Timestamp;
//import java.util.Time;
//import java.util.Date;

/**
 * A class holding miscellaneous "Utility" methods that more than 1 programmer
 * may need
 * @author Ben
 */
public class Utils {
	static boolean debugging = true;
	/**
	 * Note: If you need a calendar containing the current time, just use the constructor
	 * eg new GregorianCalendar() and that'll work.  No such method in Utils.
	 */
	
	/**
	 * @return	A SQL Date object holding date information about the current date
	 * 			(lowest granularity = day)
	 */
	public static Date getDate() {
		GregorianCalendar gregCal = new GregorianCalendar();
		long millisSince1970 = gregCal.getTimeInMillis();
		return new Date(millisSince1970);
	}
	
	/**
	 * @return	A SQL Time object holding time information about the current time
	 * 			(highest granularity is hour)
	 */
	public static Time getTime() {
		GregorianCalendar gregCal = new GregorianCalendar();
		long millisSince1970 = gregCal.getTimeInMillis();
		return new Time(millisSince1970);
	}
	
	/**
	 * @return	A SQL Timestamp object holding both date and time information
	 */
	public static Timestamp getTimeStamp() {
		GregorianCalendar gregCal = new GregorianCalendar();
		long millisSince1970 = gregCal.getTimeInMillis();
		return new Timestamp(millisSince1970);
	}

	public static int getCurrentYear() {
    	GregorianCalendar cal = new GregorianCalendar();
    	int year = cal.get(Calendar.YEAR);
    	return year;
	}
	
	public static int getCurrentMonth() {
    	GregorianCalendar cal = new GregorianCalendar();
    	int month = cal.get(Calendar.MONTH);
    	return month;
	}
	
	public static int getCurrentDayOfMonth() {
		GregorianCalendar cal = new GregorianCalendar();
    	int day = cal.get(Calendar.DAY_OF_MONTH);
    	return day;
	}
	
	// from 0-11
	public static String getMonthName(int month) {
		// There's gotta be a better way than this but I can't find a method for it
		String monthName = null;
		switch (month) {
		case Calendar.JANUARY:
			monthName = "January";
			break;
		case Calendar.FEBRUARY:
			monthName = "February";
			break;
		case Calendar.MARCH:
			monthName = "March";
			break;
		case Calendar.APRIL:
			monthName = "April";
			break;
		case Calendar.MAY:
			monthName = "May";
			break;
		case Calendar.JUNE:
			monthName = "June";
			break;
		case Calendar.JULY:
			monthName = "July";
			break;
		case Calendar.AUGUST:
			monthName = "August";
			break;
		case Calendar.SEPTEMBER:
			monthName = "September";
			break;
		case Calendar.OCTOBER:
			monthName = "October";
			break;
		case Calendar.NOVEMBER:
			monthName = "November";
			break;
		case Calendar.DECEMBER:
			monthName = "December";
			break;
		default:
			Utils.assertNotReached();
		}
		return monthName;
	}
	
	/**
	 * Toggles the debugging status of the application.
	 */
	public static void toggleDebugging() {
		Utils.debugging = !Utils.debugging;
	}
	
	/**
	 * Assertion method; All assertions can be enabled/disabled by toggling
	 * the global debugging variable
	 */
	public static void assertTrue(boolean condition) {
		if (debugging && !condition) {
			Log.e("Utils.java", "****Failed assertion!!!!");
			throw new AssertionFailureError();
		}
	}
	
	public static void assertTrue(boolean condition, String message) {
		if (debugging && !condition) {
			Log.e("Utils.java", "****Failed assertion!!!!:" + message);
			throw new AssertionFailureError(message);
		}
	}
	
	public static void assertFalse(boolean condition) {
		assertTrue(!condition);
	}
	
	public static void assertFalse(boolean condition, String message) {
		assertTrue(!condition, message);
	}

	public static void assertNotReached() {
		assertTrue(false);
	}
	
	/**
	 * Allows us to create stack traces without actually running into an 
	 * exception or dumping the stack (invisible at console because of Android)
	 * Note that normally this doesn't have to be called.  Android logs
	 * most exceptions in Logcat's Error filter.
	 * 
	 * @return	a String representing the current stack trace of the application
	 */
	public static String getStackTrace() {
		Exception aThrowable = new Exception();
		final Writer result = new StringWriter();
	    final PrintWriter printWriter = new PrintWriter(result);
	    aThrowable.printStackTrace(printWriter);
	    return result.toString();
	}
	
	public static GregorianCalendar cloneGregorianCalendar(GregorianCalendar cal) {
		GregorianCalendar calendarToReturn = new GregorianCalendar();
		calendarToReturn.set(cal.get(GregorianCalendar.YEAR),
							cal.get(GregorianCalendar.MONTH),
							cal.get(GregorianCalendar.DAY_OF_MONTH));
		return calendarToReturn;
	}
}

/**
 * Thrown when an assertion fails.
 */
class AssertionFailureError extends Error {
    AssertionFailureError() {
		super();
    }

    AssertionFailureError(String message) {
		super(message);
    }
}
